#!/bin/bash

[ -z "$LOG"] && LOG="output.log"

# Helpers
Message()
{
	kdialog --msgbox "$1"
}

DCOPHelper()
{
        echo "Number = $#"

        case $# in
        2)
        dcop $DIALOGID KommanderIf "$2" "$1" 2>&1
        ;;
        3)
        dcop $DIALOGID KommanderIf "$2" "$1" "$3" 2>&1
        ;;
        4)
        dcop $DIALOGID KommanderIf "$2" "$1" "$3" "$4" 2>&1
        ;;
        5)
        dcop $DIALOGID KommanderIf "$2" "$1" "$3" "$4" "$5" 2>&1
        ;;
        6)
        dcop $DIALOGID KommanderIf "$2" "$1" "$3" "$4" "$5" "$6" 2>&1
        ;;
        esac
}

SetProject()
{
 {
        PROJECTNAME="New$1"
        
        DCOPHelper ProjectName setText "$PROJECTNAME" &
 }  >>$LOG 2>&1
}

Status()
{
	DCOPHelper StatusBar setText "$1"
}

# Init/Destroy
Init()
{
        if [ "$LOG" != "/dev/null" ]
        then
                >$LOG
                xterm +sb -vb -e "tail -f $LOG" &
                LOG_PID=$!
                echo "$LOG_PID" >LOG_PID
        fi
 {
        DCOPHelper ProjectFolder setText "$HOME/Desktop"
        
        echo "$HOME/Desktop added"

        for FOLDER in *
        do
		if [ -n "`ls $FOLDER/*.desktop 2>/dev/null`" ]
		then
                        DCOPHelper ProjectList insertItem "$FOLDER" 0
                fi
        done
 }  >>$LOG 2>&1
}

Destroy()
{
        if [ -e LOG_PID ]
        then
                LOG_PID="`< LOG_PID`"
                rm -f LOG_PID
                kill $LOG_PID
        fi
}

# Event Handlers
CreateProject()
{
 {
        PROJECTNAME=$1
        PROJECTFOLDER=$2
        PROJECTTEMPLATE=$3
	
	TARGETFOLDER="$PROJECTFOLDER/$PROJECTNAME"
    
        mkdir -p "$TARGETFOLDER"
        
        cp -d "$PROJECTTEMPLATE"/* "$TARGETFOLDER/"
        
        if pushd "$TARGETFOLDER"
    then
        for FILE in "$PROJECTTEMPLATE"*
        do
            EXTENSION="`echo "$FILE" |egrep -o "\.[^.]*$"`"
            
    	    mv $FILE "$PROJECTNAME"$EXTENSION
        done

            find -type f -exec sed -i 's/'"$PROJECTTEMPLATE"'/'"$PROJECTNAME"'/g' {} \;
            sed -i '/^Exec/cExec=cd '"$TARGETFOLDER"' && ./'"$PROJECTNAME"'.sh --GUI' "$PROJECTNAME.desktop"
                
                sudo chown -R `id -un`:`id -gn` .
                chmod -R u+w .

                find -type d -name ".svn" -exec rm -rf {} \;

            popd
    
            Status "Created project $PROJECTNAME in folder $PROJECTFOLDER"

	    nedit "$TARGETFOLDER/$PROJECTNAME.sh" &
	    kmdr-editor  "$TARGETFOLDER/$PROJECTNAME.kmdr" &
		dcop $DIALOGID MainApplication-Interface quit
    else
            Status "Project $PROJECTNAME NOT created in folder $PROJECTFOLDER"
        fi
 }  >>$LOG 2>&1
}

# Use this as the main entry point
if [ -z "$DIALOGID" ]
then
        export ARGV1=$1
        export ARGV2=$2
        
    if [ "$1" == "--GUI" ] || [ "$1" == "--DEBUG" ]
    then
                [ "$1" == "--GUI" ] && export LOG="/dev/null"
        kmdr-executor CreateProject.kmdr
    else
        if [ $# -lt 2 ]
        then
            echo "USE: $0 <project name> <project folder>"
            echo "or run GUI by:"
            echo "$0 --GUI"
            exit
        fi
    
        CreateProject $1 $2
    fi
fi
