#!/bin/bash

CDMOUNT="cdmount"
EXTRACTFOLDER="squashfs"
DISCIMAGE="`pwd`/disc.img"
UPDATEPERIOD=1
LOG="output.log"
DISCMOUNT=discmount
SQUASHFS=casper/filesystem.squashfs
DISCSIZE=2k

Progress()
{
	dcop $DIALOGID KommanderIf setText ExtractProgress "$1"
}

Status()
{
	dcop $DIALOGID KommanderIf setText StatusBar "$1"
}

TimeProgress()
{
	local THISPID=$1
	PERCENT=0
	
	while [ -n "`ps hp "$THISPID"`" ] && [ $PERCENT -lt 100 ]
	do
		Progress $PERCENT
		PERCENT=$(( $PERCENT + 4 ))
		sleep $UPDATEPERIOD
	done

	Progress 99

	while [ -n "`ps hp "$THISPID"`" ] 
	do
		sleep $UPDATEPERIOD
	done
	Progress 100
}

WatchFileProgress()
{
	local THISPID=$1
	local THISFILE="$2"
	local THISFILEFULLSIZE=$3
	
	while [ -n "`ps hp "$THISPID"`" ]
	do
		SIZE="`ls -l "$THISFILE" |awk '{print $5}' 2>>$LOG`"
		PERCENT=$(( $SIZE * 100 / $THISFILEFULLSIZE ))
		Progress $PERCENT
		sleep $UPDATEPERIOD
	done
	Progress 100
}

WatchCommandProgress()
{
	local THISPID=$1
	local THISCOMMAND="$2"
	local THEFIELD="$3"
	local THEFULLSIZE=$4
	
	while [ -n "`ps hp "$THISPID"`" ]
	do
		SIZE="`$THISCOMMAND |cut -f$THEFIELD 2>>$LOG`"
#Status "SIZE=$SIZE $THISCOMMAND=`$THISCOMMAND`"
		PERCENT=$(( $SIZE * 100 / $THEFULLSIZE ))
		Progress $PERCENT
		sleep $UPDATEPERIOD
	done
	Progress 100
}

WatchDeviceProgress()
{
	local THISPID=$1
	local THISMOUNT="$2"
	
	while [ -n "`ps hp "$THISPID"`" ]
	do
		SIZE="`df -P $THISMOUNT |awk 'END {sub (/%/, "", $5); print $5}' 2>>$LOG`"
#Status "SIZE=$SIZE"
		Progress $SIZE
		sleep $UPDATEPERIOD
	done
	Progress 100
}

# Message handler for the "Say Hello..." Button
ProgressAndStatus()
{
	CDIMAGE=$1
	
	Status "Accessing $CDIMAGE content"	
	mkdir -p $CDMOUNT	
	sudo mount -o loop -t iso9660 "$CDIMAGE" "$CDMOUNT" >>$LOG 2>&1
	
	Status "Creating new disc image"
	nice dd if=/dev/zero of=$DISCIMAGE bs=1M count=$DISCSIZE &
	PID=$!
	WatchFileProgress "$PID" "$DISCIMAGE" 2147483648
	
	Status "Formatting and accessing, please wait"
	/sbin/mke2fs -F $DISCIMAGE >>$LOG 2>&1 &
	PID=$!
	TimeProgress $PID
	mkdir -p $DISCMOUNT	 >>$LOG 2>&1
	sudo mount -o loop -t ext2 $DISCIMAGE $DISCMOUNT >>$LOG 2>&1
	
	Status "Extracting filsystem"
	nice sudo /usr/sbin/unsquashfs -dest $DISCMOUNT/rootfs  $CDMOUNT/$SQUASHFS >>$LOG 2>&1 &
	PID=$!
	UPDATEPERIOD=1
#	WatchCommandProgress $PID "du -s $DISCMOUNT" 1 2147483
	WatchDeviceProgress $PID $DISCMOUNT
	sudo mv $DISCMOUNT/rootfs/* $DISCMOUNT/ >>$LOG 2>&1 &
	
	Status "Finishing"
	sync
	sudo umount "$DISCMOUNT" >>$LOG 2>&1
	sudo umount "$CDMOUNT" >>$LOG 2>&1
	rmdir "$DISCMOUNT" "$CDMOUNT" >>$LOG 2>&1
	
#	kdialog --textbox $LOG 400 400
}

# Use this as the main entry point
if [ -z "$DIALOGID" ]
then
    if [ "$1" == "--GUI" ]
    then
    	kmdr-executor ProgressAndStatus.kmdr
    else
        if [ $# -lt 1 ]
        then
            echo "USE: $0 <path to CD image>"
            echo "or run GUI by:"
            echo "$0 --GUI"
            exit
        fi
    
        ProgressAndStatus $1
    fi
fi
